import os

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "netfields",
    }
}

INSTALLED_APPS = (
    "django.contrib.postgres",
    "netfields",
    "test",
)

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
)

SECRET_KEY = "notimportant"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"
